/********************************************************************************************************************** 
Macro name : PSBJNPCT1
Project/Study : Applicable to any project/study
Description : With ADaM OCCDS as input, any hierarchy freq tables can be generated, e.g., tables of MH, CM, CE, and AE. 
              For some simple summary tables like MH or CE, SDTM can be as input dataset. 
Output : Any Hierarchy Frequency Tables with RTF format
Author : Chengxin Li
SAS version/platform : SAS 9.x with Windows or EG with Linux 
Parameter Definitions : Refer to PhamsSUG 2021 paper AD-002  
Macro Call Example : Refer to PhamsSUG 2021 paper AD-002
Modification History: Initial version Oct. 2020

DISCLAIM: ThIS SAS PROGRAM IS FOR ILLUSTRATION OF CONCEPTS PRESENTED IN THE PhamaSUG 2021 PAPER AD-002 Macro Development from Design to Implementation Applied to Occurrence Data Frequency Analysis.
          BY REFERRING TO THE CODES, READER MAY RAPIDLY APPLY THE PRESENTED CONCEPTS TO OTHER ANALYSES. HOWEVER, NEITHER BUG-FREE NOR EXECUTABLE UNDER READER'S SAS SETTING IS GUARANTEED. 

***********************************************************************************************************************/

%macro PSBJNPCT1(
    TableID=,              /*e.g., %str( Table 4.1 ), table name determined correspondingly*/                                 
	Title=,                /*separated with # if multiple*/
    Popu=SAFFL,            /*donominator, summarizing total number of subject N from ADSL*/
	Footnote=,             /*separated with # if multiple*/

	Indata=,               /*input analysis dataset; if no libname, libname WORK by default; TRTA/TRTAN required or TRTP/TRTPN cond used*/
    WHERE=1,               /*WHERE criteria to select data, 1 by default, typically including popu, timepoints, analysis flag such as TRTEMFL*/
	SubGroup=,             /*subgroup analysis variable; if provided, existences required both in ADaM and ADSL*/
    Analvar=,              /*hierarchy analysis variables; separated with # if multiple; four order options defined for the associated analysis variable, e.g.,ASEV@preloadfmt=$AESEV.*/
    ADSL_TRT=%str(TRT01A/TRT01AN), /*ADSL treatment pair variables, separated with /. If NULL, by default, TRT01A/TRT01AN used */
	Total= ,               /*Total or subtotal column for all or sub treatemnt groups; disnabled by default; and enabled by Y, or Yes; Subtotal enabled by selected ones from TRTA and separated with #*/
	FreqThreshold=         /*Frequency threshold cut-off; only exceeding the threshold included (e.g., 0.05, i.e., >5%) listed within any treatment group*/
);

    /******************************************************************************
    *******************************************************************************
    ************************ BLOCK 0: INITIALIZATION ******************************
    ******************************************************************************/
	
	/***RTF template defined. This template was modifiled from examples in public domains***/
	proc template;           

      define style Styles._MARTF;

  	  parent = Styles.RTF;

	  replace fonts /
		"TitleFont" = ("TimesNewRoman",12pt,Bold)  
		"TitleFont2" = ("TimesNewRoman",12pt,Bold) 
		"StrongFont" = ("TimesNewRoman",10pt,Bold) 
		"EmphasisFont" = ("TimesNewRoman",12pt)    
		"headingEmphasisFont" = ("TimesNewRoman",11pt)
		"headingFont" = ("TimesNewRoman",11pt) 	   
		"docFont" = ("TimesNewRoman",10pt)        
		"footFont" = ("TimesNewRoman",10pt)        
		"FixedEmphasisFont" = ("Courier",9pt)
		"FixedStrongFont" = ("Courier",9pt,Bold)
		"FixedHeadingFont" = ("Courier",9pt,Bold)
		"BatchFixedFont" = ("Courier",6.7pt)
		"FixedFont" = ("Courier",9pt);

      replace color_list /
		"link" = blue 
 		"bgH" = white 
		"fg" = black  
		"bg" = white;

      replace Body from Document /  
		bottommargin = 0.75in
		topmargin = 1in
		rightmargin = 1in
		leftmargin = 1in;

      replace Table from Output /
	  	outputwidth=80%  
		frame = hsides    
		rules = groups    
		cellpadding = 3pt 
		cellspacing = 0pt 
		borderwidth = 1pt;

      style systemTitle from systemTitle /
	    just=left
        protectspecialchars=off; 

	  style SystemFooter from SystemFooter /
	    just=left
		font = fonts("footFont");
      /*  protectspecialchars=off; */ 

    end;
    run;
  

	/**************************************************************
	**** system option settings **********************************/

  	%if %symexist(debug) %then %do;
   		%let debug=&debug.;
		%if %upcase(&debug.)=Y or %upcase(&debug.)=YES %then %do;
			options fullstimer;
			options nomprint mlogic symbolgen;
        %end;
		%else %do;
			options nofullstimer;
			options nomprint nomlogic nosymbolgen;
		%end;
   	%end;

  	options PAPERSIZE=(11.69in 8.27in); 
	%if %symexist(papersize) %then %do; 
		%if %length(&papersize.) %then options papersize=&papersize.;
		%else options PAPERSIZE=(11.69in 8.27in);;
	%end;
  	/*proc options option=papersize;
	run;*/
 
	options nodate nonumber nobyline; 
	options FORMCHAR="|----|+|---+=|-/\<>*";
	options missing='';
	options nosource;
	options linesize=max;

	ods escapechar='^';
	ods html close;
	ods listing close;
    ods results=off;


	/**********************************************************************
	******taking snapshot for datasets in WORK libref at initiations******/

	data snapshot_ds;
		set sashelp.vtable;
		where libname='WORK' and memtype="DATA";
		keep libname memname memtype;
	run;

   	%let dsid=%sysfunc(open(work.snapshot_ds)); 
    %let dsn=%sysfunc(attrn(&dsid,nobs));     
    %let rc=%sysfunc(close(&dsid));
	%if &dsn.>0 %then %do;          
		proc sql noprint;
			select distinct memname into: dslist separated by ' '
			from snapshot_ds;
		quit;
	%end;

	/**************************************************************
	**** sub-macro within PSBJNPCT1 ********************************/

	%local i j k;

	/*define Dir exists macro. This sub-macro was from public SAS community.*/
	%macro DirExist(dir) ; 
   		%LOCAL rc fileref return; 
   		%let rc = %sysfunc(filename(fileref,&dir)); 
   		%if %sysfunc(fexist(&fileref))  %then %let return=1;    
   		%else %let return=0;
   		&return
	%mend DirExist;

    /******************************************************************************
    ******************************** END OF BLOCK 0 *******************************
    *******************************************************************************
    ******************************************************************************/

    /************************************************************************************
    *************************************************************************************
    *BLOCK 1: PROCESS LIBREF, FILE PATH, MACRO PARAMETER INPUTS, GETTING ANALYSIS READY *
    ************************************************************************************/

	/******************************************************************************
    ************** LIBREF, FILEREF PATH, table ID, and  also RDS ******************
    ******************************************************************************/
	%if &sysscp=WIN %then %do;  
  		%let progpathnm = %SYSGET(SAS_EXECFILEPATH); 
 		%let prognm =%SYSGET(SAS_EXECFILENAME);
  		%let progpath =%SUBSTR(&progpathnm.,1,%eval(%length(&progpathnm)-%length(&prognm)));     

		%let progpath_temp=%SUBSTR(&progpath.,1,%length(&progpath.)-1);  
		%let j=%sysfunc(findc(&progpath_temp., \, B));
		%let progUp=%SUBSTR(&progpath_temp.,1,&j);   
  		%let outputpath=&progUp.Outputs;            
	%end;
	%else %do;										 
		%if %symexist(outputpath) %then %let outputpath=&outputpath.;
		%else %do;
			%put %str(E)RROR: The output path is required to users. Since the outputpath not provided, the macro call instance halted.; 
      		%goto exit;
		%end; 
	%end; 

	%let outputrc=%DirExist(&outputpath.); 
	%if &outputrc.=1 %then %do;            
		%if &sysscp=WIN %then %let rtfpath=&outputpath.%str(\); 
		%else %do;                                                 
			%let lastchar=%SUBSTR(&outputpath.,%length(&outputpath.),1);
			%if "&lastchar"="/" %then %let rtfpath=&outputpath.; %else %let rtfpath=&outputpath.%str(/);
		%end;
		%put %str(N)OTE: Produced RTF table saved in existed &outputpath.; 
	%end;
	%else %do;                             
		%if &sysscp=WIN %then %do;
			%put %str(N)OTE: Outputs folder parallel with program resided not existed, thus produced RTF table saved in &progpath.;  
			%let rtfpath=&progpath.;
		%end;
		%else %do;
			%put %str(E)RROR: The output path provided not existed, the macro call instance halted.; 
      		%goto exit;
		%end;
	%end;

	%let tableid=&tableid.;
	%if %length(&tableid.) %then %do; 
	    %let tableid=%sysfunc(compbl(&tableid.));
    	%let tableNM0=%sysfunc(translate(&tableid., '_','.')); 

        %if %upcase(%substr(&tableNM0.,1,1))=T %then %let tableNM=%sysfunc(translate(&tableNM0., '_',' '));
		%else %let tableNM=t_&tableNM0.;

		%let tableNM=%sysfunc(lowcase(&tableNM.));
		%let tableNM=%sysfunc(tranwrd(&tableNM.,table,t)); 
	%end;
    %else %do; 
		%let tableid=Table 99;
		%let tableNM=t_99; 
		%put %str(W)ARNING: Parameter TableID not provided but recommended, the output table name assigned as t_99 correspondingly; 
	%end;

	filename logfile "&rtfpath.&tableNM..log";
	proc printto log=logfile new; 
	run;

  	%let rdsrc=%sysfunc(libref(RDS));
    %if &rdsrc.=0 %then %put %str(N)OTE: RDS libref available, RDS dataset for the table automatically saved in RDS libref location.; 
    %else %do;                                 
		%if &outputrc.=1 %then %do;         
			%put %str(N)OTE: RDS(Table Result Dataset) libref not available, RDS dataset for the table automatically saved in existed &outputpath.; 
			%macro opc;
				libname rds "&outputpath.";
			%mend opc;	
			%opc;
			/*libname RDS "&outputpath.";*/
		%end;
		%else %do;							   
			%put %str(N)OTE: Neither RDS(Table Result Dataset) libref nor Outputs folder available,RDS dataset for the table automatically saved in &progpath.;
 
			%macro ppc;
				libname rds "&progpath.";
			%mend ppc;	
			%ppc;
			/*libname RDS "&progpath.";*/
		%end;
	%end;;


	/******************************************************************************
    *** get ADSL and input analysis dataset var list ******************************
    ******************************************************************************/

	proc sql;
		create table ADSL_loc as
		select distinct libname, memname, crdate
		from sashelp.vtable
		where memname='ADSL'
		order by crdate;
	quit;

   	%let dsid=%sysfunc(open(work.ADSL_loc)); 
    %let n=%sysfunc(attrn(&dsid,nobs));     
    %let rc=%sysfunc(close(&dsid));         

    %if &n=0 %then %do; 
		%put %str(E)RROR: ADSL is not located. Please check if LIBREF assigned successfully. The macro call instance halted.;
		%goto exit;
	%end;

	data _null_;          
	  set adsl_loc;
	  by memname crdate;
	  if last.memname;

	  call symputx('LIBNM', libname);
	run;
	%if %length(&LIBNM.) %then %do;
    	%let ADSL=&LIBNM..ADSL; 
		%put %str(N)OTE: &ADSL. is used in the macro call instance.; 
	%end;

	/*optionally from DICTIONARY.COLUMNS to get variable list*/
   	proc contents data=&ADSL out=adsl_vars(keep=varnum name label) noprint;
   	run;
   	proc sql noprint;
    	select distinct upcase(name) into: ADSLVARs separated by ' '
     	from adsl_vars;
   	quit;
 
	%let INDATA=&INDATA.;

	%if %length(&INDATA.)=0 %then %do;
	  %put %str(E)RROR: &INDATA. not provided, the macro call instance halted.; 	
      %goto exit;
	%end;

   	%if %sysfunc(indexc(&INDATA.,%str('.'))) %then %do;  
     	%let ds =%scan(&INDATA, -1, '.');
		%let lib=%scan(&INDATA, 1, '.');
		%if %upcase(&lib.)^=%str(WORK) %then %do;
			skip 1;
			%put %str(N)OTE: Downloading &DS. from &lib. library into WORK library...; 
			proc copy in=&lib. out=work;
				select &ds.;
			run;
			%put %str(N)OTE: &DS. download completed.;
			%put ;
		%end;
		%else %let ds = &INDATA.;    
   	%end; 
	%else %do;                      
    	%let ds = &INDATA.;
  	%end;

    
	/*** optionally with SAS native macro &sqlOBS to retrive number of variables, e.g., %let DSVARN=&sqlOBS. ***/
   	proc contents data=&DS. out=DS_vars(keep=varnum name label) noprint;
   	run;
   	proc sql noprint;
    	select distinct upcase(name), max(varnum) into: DSVARs separated by ' ', 
                                                      : DSVARN
     	from DS_vars;
   	quit;  

	/*********************************************************************************
    *******checking variable requirements with INDATA:Treatment*********************** 
    *********************************************************************************/
  
	%let ADSL_TRT=%upcase(&ADSL_TRT.);

	%if %length(&ADSL_TRT)=0 %then %do;
	 	%let TRTxxA=TRT01A;
		%let TRTxxAN=TRT01AN;
	  	%put %str(N)OTE: ADSL treatment variables not provided. By default, ADSL.TRT01A/TRT01AN used.; 	
	%end;

	%if %length(&ADSL_TRT)>0 %then %do;
		%if %sysfunc(indexc(&ADSL_TRT.,%str('/')))=0 %then %do;
	  		%put %str(E)RROR: ADSL treatment pair(treatment and treatment order) variables separted with '/' required for ADSL_TRT parameter, e.g., TRT01A/TRT01AN.; 	
      		%goto exit;
		%end;
		%else %do;
		   %let TRTxxA=%scan(&ADSL_TRT., 1, '/');              
		   %let TRTxxAN=%scan(&ADSL_TRT., -1, '/');           
		%end;
    %end;

  	%if %sysfunc(indexw(&ADSLVARs.,&TRTxxA.))=0 %then %do;
	  	%put %str(E)RROR: &TRTxxA. not existed in ADSL but required for analysis, consequently the macro call instance halted.; 	
      	%goto exit;
	%end;
	%else %do;   
	    %if %upcase(%substr(&TRTxxA.,%length(&TRTxxA),1))=P %then %do;
	        %put;
			%put %str(W)ARNING: Actual treament variable should be used. If TRTxxP used, please check and make sure TRTxxP=TRTxxA in the study.; 
            %put;
		%end;

     	%if %sysfunc(indexw(&ADSLVARs.,&TRTxxAN.))=0 %then %do;
		  	%put %str(E)RROR: &TRTxxAN. not existed in ADSL but also required along with &TRTxxA. for analysis, consequently the macro call instance halted.; 	
    	  	%goto exit;		
    	%end;
		%else %put %str(N)OTE: Input &TRTxxA./&TRTxxAN. in ADSL are used in the anlysis.; 	

   	%end;
	

    %let TRTVAR=;
	%let TRTNVAR=;

	%if %length(&DS.)>2 %then %do;                   

  		%if %sysfunc(indexw(&DSVARs.,TRTA))=0 %then %do; 
			%if %sysfunc(indexw(&DSVARs.,TRTP))=0 %then %do; 
	  			%put %str(E)RROR: Neither TRTA nor TRTP existed but either one required for the analysis, consequently the macro call instance halted.; 	
      			%goto exit;
			%end;
			%else %do;      
				%if %sysfunc(indexw(&DSVARs.,TRTPN))>0 %then %do; 
					%let TRTVAR=TRTP;
					%let TRTNVAR=TRTPN;
					%put %str(W)ARNING: TRTA/TRTAN (highly recommended) not available but TRTP/TRTPN existed in %upcase(&INDATA.), TRTP/TRTPN are conditionally used in the analysis.;
					%put %str(W)ARNING: Please check and make sure TRTP=TRTA in the study.; 
				%end;
				%else %do; 
	  				%put %str(E)RROR: TRTA/TRTAN not available in %upcase(&INDATA.). TRTP existed but TRTPN not existed. Consequently the macro call instance halted.; 
      				%goto exit;
				%end;

			%end;

		%end;
    	%else %do; 
			%if %sysfunc(indexw(&DSVARs.,TRTAN))>0 %then %do; 
				%let TRTVAR=TRTA;
				%let TRTNVAR=TRTAN;
				%put %str(N)OTE: TRTA/TRTAN in %upcase(&INDATA.) are used in the analysis.; 
			%end;
			%else %do; 
	  			%put %str(E)RROR: Not only TRTA but also TRTAN required for the analysis. TRTAN not existed in %upcase(&INDATA.), consequently the macro call instance halted.; 	
      			%goto exit;
			%end;
		%end;

	%end;	
	%else %do;      
		%let TRTVAR=&TRTxxA;
		%let TRTNVAR=&TRTxxAN;
	%end;

	/******************************************************************************
    ***process parameters  title, footnote, popu, SubGroup, analval           ***** 
	*** WHERE not checked and leaving to SAS, by default set WHERE=1 though *******
    ******************************************************************************/
 
   	%let title=&title.;
	%if %length(&title.) %then %do;
	    %let title=%sysfunc(compbl(&title.));
        %let tn=%qsysfunc(countc(&title.,%str(#)));            
		
		%do i=1 %to %eval(&tn.+1);                      
            %let titl&i.=%scan(&title.,&i.,%str(#)); 
		%end;
         
	%end;
	%else %do; 
	  %put %str(E)RROR: Parameter TITLE not provided but required, the macro call instance terminated.; 
	  %goto exit;
	%end;


   	%let footnote=&footnote.;
	%if %length(&footnote.) %then %do;
	    %let footnote=%sysfunc(compbl(&footnote.));
        %let fn=%qsysfunc(countc(&footnote.,%str(#)));           
		
		%do i=1 %to %eval(&fn.+1);   
            %let foot&i.=%scan(&footnote.,&i.,%str(#));
		%end;
         
	%end;
	%else %do; 
	  %put %str(N)OTE: User footnote not provided in the macro call instance.; 
	%end;
  

	%let popuvar=%upcase(&popu);

	%if %length(&popuvar.)=0 %then %do;
	  %let popuvar=SAFFL;
	  %put %str(N)OTE: Parameter POPUVAR not provided. By default, SAFFL used in the macro call instance.; 	
    %end;

	%if %sysfunc(indexw(&ADSLVARs.,&popuvar.))=0 %then %do;
	  %put %str(E)RROR: &POPUVAR. not existed in ADSL, the macro call instance halted.; 	
      %goto exit;
	%end;
	%else %do;
		%let adsl_popu=&popuvar.;
	    %put %str(N)OTE: ADSL.&POPUVAR. used in the analysis.; 	
	%end;

	%if %length(&DS.)>2 %then %do;    
		%if %sysfunc(indexw(&DSVARs.,&popuvar.))=0 %then %do;    
			%let ds_popu=1;
			%put ;
	  		%put %str(W)ARNING: &POPUVAR. not existed in %upcase(&INDATA.), correspondingly all subjects in %upcase(&INDATA.) selected for the analysis.;
			%put ;
		%end;
		%else %do;
			%let ds_popu=&popuvar.;
	    	%put %str(N)OTE: &DS..&POPUVAR. used in the analysis.; 	
		%end;
	%end;
	%else %do;   
	  	%put %str(N)OTE: &POPUVAR. not applied to SDTM domain as input dataset, correspondingly all subjects in %upcase(&INDATA.) selected for the analysis.;
		%let ds_popu=1;
	%end;

	%let where=%upcase(&where.);

	%if %length(&where.)=0 %then %do;
	  %let where=1;
	  %put %str(N)OTE: User WHERE parameter not provided, thus assign WHERE=1 by default for the macro call instance.; 	
    %end;


	%let subGroup=%upcase(&SUBGROUP.);
	%if %length(&subGroup.) %then %do;
		%if %sysfunc(indexw(&DSVARs.,%upcase(&subGroup.)))=0 or %sysfunc(indexw(&ADSLVARs.,%upcase(&subGroup.)))=0 %then %do; 
	  		%put %str(E)RROR: The provided &subgroup. not existed in either %upcase(&INDATA.) or ADSL, the macro call instance halted.; 	
      		%goto exit;
		%end;
	%end;


	%let ANALVAR=&ANALVAR.;

	%let ANALVAR_str=;

    %if %length(&ANALVAR.) %then %do;                   

 		%let an=%sysfunc(countc(&ANALVAR.,'#'));        

		%let sqls_str=;
		%let sqlby_str=;

		%let fmt_str=;    
		%let by_str=;     
		%let class_str=;  
		%let typ_str=;    

		%do j=1 %to %eval(&an.+1); 

			%let fmt_str&j.=&fmt_str.;
			%let by_str&j.=&by_str.;			
			%let class_str&j.=&class_str.; 
			%let typ_str&j.=&typ_str.;

			%let sqls_str&j.=&sqls_str.;
			%let sqlby_str&j.=&sqlby_str.;

            %let ANALVAR&j.=%scan(&ANALVAR.,&j.,%str(#));              

          	%if %sysfunc(indexc(&&ANALVAR&j.,%str('@')))>0 %then %do;  
			    %let aval&j=%scan(&&ANALVAR&j., 1, '@');               
			    %let fmtstr&j.=%scan(&&ANALVAR&j., -1, '@');           

				%if %sysfunc(indexw(&DSVARs.,&&aval&j.))=0 %then %do;  
	 				 %put %str(E)RROR: Analysis variable &&aval&j. not existed in %upcase(&INDATA.), the macro call instance halted.; 	
      			     %goto exit;
			    %end;

				%if %sysfunc(indexc(&&fmtstr&j.,%str('=')))>0 %then %do;  
                	 %let fmtwd&j=%upcase(%scan(&&fmtstr&j., 1, '=')); 
					 %let fmt&j=%scan(&&fmtstr&j., -1, '=');

					 %if &&fmtwd&j.^=%str(PRELOADFMT) and &&fmtwd&j.^=%str(ORDER) and &&fmtwd&j.^=%str(INFORMAT) and &&fmtwd&j.^=%str(FREQ) %then %do;
	 				 	%put %str(E)RROR: For &&aval&j., the sort option requires PRELOADFMT, ORDER, INFORMAT, FREQ. The macro call instance halted.; 
      			     	%goto exit;
					 %end;

					 %if &&fmtwd&j.=%str(PRELOADFMT) %then %do;  

					 	%if %sysfunc(indexc(&&fmt&j.,%str(.)))=0 %then %let fmt1&j=&&fmt&j..%str(.);  
					 	%else %let fmt1&j.=&&fmt&j.;

					 	%let fmt2&j=%upcase(%sysfunc(compress(%scan(&&fmtstr&j., -1, '='), %str(.)))); 


	  					proc sql noprint;
   							create table fmt_temp as 
   							select libname, fmtname, source, fmttype
   							from dictionary.formats
  							where fmtname="&&fmt2&j" and source='C'; 
   	  					quit;

   	  					%let dsid=%sysfunc(open(work.fmt_temp)); 
      					%let n=%sysfunc(attrn(&dsid,nobs));     
      					%let rc=%sysfunc(close(&dsid));         

      					%if &n=0 %then %do;
							%put %str(E)RROR: The completetype with provided preloadfmt format for &&aval&j. not existed. The macro call instance halted.;
							%goto exit;
						%end;
						%else %do;  
							%let fmt_str&j.=&&fmt_str&j. &&aval&j. &&fmt1&j.;
							%let class_str&j.=&&class_str&j. &&aval&j.;
							%if %length(&&typ_str&j.) %then %let typ_str&j.=&&typ_str&j. %str(*) &&aval&j.; %else %let typ_str&j.=&&aval&j.;

							
							%let fmti&j.=%sysfunc(compress(&&fmt2&j., %str($)));		                	
	
							proc format library=work cntlout=work.preloadfmt(where=(FMTNAME="&&fmti&j." and TYPE='C'));
							quit;	

							data &&fmti&j..NP;
								set preloadfmt(keep=FMTNAME START END LABEL);
								FMTNAME="&&fmti&j..NP";
								LABEL=_n_;
								TYPE='I';
								output;
							run;

							proc sort data=&&fmti&j..NP nodup;
	    						by LABEL;
	  						run;
	  
	  						proc format lib=work.formats CNTLIN=&&fmti&j..NP;
	  						run;
	  						quit;

							%let AVALP&j.=&&fmti&j..NP;

						%end;

					 %end;  
					 %else %if &&fmtwd&j.=%str(ORDER) %then %do; 
					 	%let fmt&j.=%upcase(&&fmt&j.);
						%if %sysfunc(indexw(&DSVARs.,&&fmt&j.))=0 %then %do;  
	 				 		%put %str(E)RROR: The denoted sort variable &&fmt&j. not existed in %upcase(&INDATA.). The macro call instance halted.; 	
      			     		%goto exit;
						%end;
						%else %do; 
							%let AVALP&j.=;       
							%let avalN&j.=&&fmt&j.; 
	 		    			%put %str(N)OTE: The denoted sort variable &&fmt&j. existed in %upcase(&INDATA.), the table display for &&aval&j. sort by variable &&fmt&j..; 
						%end;

						%if %length(&&by_str&j.) %then %let sqlby_str&j.=&&sqlby_str&j. %str(,) &&aval&j.; %else %let sqlby_str&j.=&&aval&j.; 
						%let by_str&j.=&&by_str&j. &&aval&j.;  

					 %end;  
					 %else %if &&fmtwd&j.=%str(INFORMAT) %then %do; 
					 	%if %sysfunc(indexc(&&fmt&j.,%str(.)))=0 %then %let fmt1&j=&&fmt&j..%str(.);  
					 	%else %let fmt1&j.=&&fmt&j.;

						%let fmt2&j=%upcase(%sysfunc(compress(%scan(&&fmtstr&j., -1, '='), %str(.)))); 

	  			    	proc sql noprint;
   							create table fmt_temp as 
   							select libname, fmtname, source, fmttype
   							from dictionary.formats
  							where fmtname="&&fmt2&j." and source='C' and fmttype="I";
   	  					quit;

   	  			 		%let dsid=%sysfunc(open(work.fmt_temp)); 
      					%let n=%sysfunc(attrn(&dsid,nobs));     
      					%let rc=%sysfunc(close(&dsid));         

      					%if &n=0 %then %do;
							%put %str(N)OTE: The denoted informat &&fmt&j. not existed for &&aval&j.. The macro call instance halted.;
							%goto exit;
						%end;
	  					%if &n>0 %then %do;
							%let AVALP&j.=;           
							%let avalN&j.=;           
							%let AVALI&j.=&&fmt1&j.;  
							%put %str(N)OTE: User denoted informat &&fmt&j. exists. The display order for &&aval&j. sort by informat &&fmt&j..;
						%end;

						%if %length(&&by_str&j.) %then %let sqlby_str&j.=&&sqlby_str&j. %str(,) &&aval&j.; %else %let sqlby_str&j.=&&aval&j.; 
						%let by_str&j.=&&by_str&j. &&aval&j.;   
 
					 %end;  
					 %else %do; 
						%if %sysfunc(compare(&&fmt&j., TOTAL, iL)) ne 0  %then %do; 
	  			    		proc sql noprint;
   								create table TRT_temp as 
   								select distinct &TRTxxA
   								from &ADSL.
  								where strip(&TRTxxA)="&&fmt&j.";
   	  						quit;

   	  			 			%let dsid=%sysfunc(open(work.TRT_temp)); 
      						%let n=%sysfunc(attrn(&dsid,nobs));     
      						%let rc=%sysfunc(close(&dsid));         

      						%if &n=0 %then %do;
								%put %str(N)OTE: The denoted treatment group not existed for &&aval&j.. The macro call instance halted.;
								%goto exit;
							%end;
	  						%if &n>0 %then %do;
								%let SORTRT&j.=&&fmt&j.;
								%let freqtotal&j.=of treatment group &&fmt&j.;
								%put %str(N)OTE: The display order for &&aval&j. sort by descending frequency &&freqtotal&j..;
							%end;		
						%end;  
						%else %do; 
							%let SORTRT&j.=Total;
							%if %sysfunc(indexc(&TOTAL.,%str('#'))) %then %let freqtotal&j.=of subtotal of %sysfunc(translate(&Total.,' ', %str(#)));
							%else %let freqtotal&j.=of Total;
								
							%put %str(N)OTE: The display order for &&aval&j. sort by descending frequency &&freqtotal&j.; 
						%end;	

						
						%let AVALP&j.=;    
						%let avalN&j.=;   
						%let AVALI&j.=;

						%if %length(&&by_str&j.) %then %let sqlby_str&j.=&&sqlby_str&j. %str(,) &&aval&j.; %else %let sqlby_str&j.=&&aval&j.; 
						%let by_str&j.=&&by_str&j. &&aval&j.;   	

					 %end;  


				%end;   
				%else %do;
	 				 %put %str(E)RROR: The expresion for sort option not conformed with /yyy=xxx for &&aval&j.. The macro call instance halted.; 	
      			     %goto exit;
				%end;
				  
				%if %length(&&sqls_str&j.) %then %let sqls_str&j.=&&sqls_str&j. %str(,) &&aval&j.; %else %let sqls_str&j.=&&aval&j.; 
				%let ANALVAR_str=&ANALVAR_str. &&aval&j.; 

            %end;  

           	%else %do;     
				%let AVALP&j.=;    

				%let ANALVAR&j.=%upcase(&&ANALVAR&j.);
				%if %sysfunc(indexw(&DSVARs.,&&ANALVAR&j.))=0 %then %do;
	 				 %put %str(E)RROR: Analysis variable &&ANALVAR&j. not existed in %upcase(&INDATA.). The macro call instance halted.; 	
      			     %goto exit;
			    %end;

				%if %length(&&by_str&j.) %then %let sqlby_str&j.=&&sqlby_str&j. %str(,) &&ANALVAR&j.; %else %let sqlby_str&j.=&&ANALVAR&j.; 
				%let by_str&j.=&&by_str&j. &&ANALVAR&j.;   

 				%let aval&j.=&&ANALVAR&j.;
				%if %length(&&sqls_str&j.) %then %let sqls_str&j.=&&sqls_str&j. %str(,) &&aval&j.; %else %let sqls_str&j.=&&aval&j.; 
				%let ANALVAR_str=&ANALVAR_str. &&ANALVAR&j.; 

                %if %length(&&ANALVAR&j.)<8 %then %do; 
					%if %sysfunc(indexw(&DSVARs.,&&ANALVAR&j..N))=0 %then %do; 
						%let avalN&j.=;
 					%end;
					%else %do;
						%let avalN&j.=&&ANALVAR&j..N;
	 		    		%put %str(N)OTE: Order variable &&ANALVAR&j..N exists in %upcase(&INDATA.). The table display for &&ANALVAR&j. sort by &&ANALVAR&j..N.; 
					%end;				 
				%end;
				%else %do;  
                    %do k=1 %to &DSVARN.;
						%let DSVAR&k.=%scan(&DSVARs.,&k.,%str( ));
						%if %length(&&DSVAR&k.)>=8 and %substr(&&DSVAR&k., 1, 6)=%upcase(%substr(&&ANALVAR&j., 1, 6)) and %substr(&&DSVAR&k., %length(&&DSVAR&k.), 1)=N %then %do;
							%let avalN&j.=&&ANALVAR&j..N;
	 		    			%put %str(N)OTE: Order variable &&ANALVAR&j..N exists in %upcase(&INDATA.). The table display for &&ANALVAR&j. sort by &&ANALVAR&j..N.; 
						%end; 
						%else %let avalN&j.=;
					%end;
				%end;

				%if %length(&&AVALN&j.)=0 %then %do; 						
	  			    proc sql noprint;
   						create table fmt_temp as 
   						select libname, fmtname, source, fmttype
   						from dictionary.formats
  						where fmtname="&&ANALVAR&j..N" and fmttype="I";
   	  				quit;

   	  			 	%let dsid=%sysfunc(open(work.fmt_temp)); 
      				%let n=%sysfunc(attrn(&dsid,nobs));     
      				%let rc=%sysfunc(close(&dsid));         

      				%if &n=0 %then %do;
						%let AVALI&j.=;

						%let SORTRT&j.=Total; 

						%if %sysfunc(indexc(&TOTAL.,%str('#'))) %then %let freqtotal&j.=of subtotal of %sysfunc(translate(&Total.,' ', %str(#)));
						%else %let freqtotal&j.=of Total;
								
						%put %str(N)OTE: Neither sort variable nor informat exists for &&ANALVAR&j.. The display sort by descending frequency &&freqtotal&j..;
					%end;
	  				%if &n>0 %then %do;
						%put %str(N)OTE: User defined informat &&ANALVAR&j..N exists. The display sort by informat &&ANALVAR&j..N for &&ANALVAR&j.;
						%let AVALI&j.=&&ANALVAR&j..N;
					%end;
				%end;
			%end;

			%let fmt_str=&&fmt_str&j.;
			%let by_str=&&by_str&j.;
			%let class_str=&&class_str&j.;
			%let typ_str=&&typ_str&j.;

			%let sqls_str=&&sqls_str&j.;
			%let sqlby_str=&&sqlby_str&j.;

		%end;  	
       
	%end;   
	%else %do;
	  %put %str(E)RROR: Parameter AnalVar not provided. The macro call instance halted.; 
	  %goto exit;
	%end;   

	%put ;
	%put %str(N)OTE: Normalized analysis variables string: &ANALVAR_str.; 
	%put ;

	/*******************************************************************************
	 *************processing TOTAL**************************************************
	 ***Note: the subtotal elements checks for correcness left to analysis block***/
	%let TOTAL=&TOTAL.;

	%if %length(&TOTAL.) %then %do;

		%if %upcase(&TOTAL.) =Y or %upcase(&TOTAL.) =YES or %sysfunc(indexc(&TOTAL.,%str('#'))) %then %put %sysfunc(compress(NOTE)): Total column explicitly enabled.;
		%else %if %upcase(&TOTAL.) =N or %upcase(&TOTAL.) =NO or %upcase(&TOTAL.) =NONE or %upcase(&TOTAL.) =NULL %then %put %sysfunc(compress(NOTE)): Total column explicitly disenabled.;
		%else %do;
			%put %str(E)RROR: Parameter TOTAL not conforming the defined rules. The macro call instance halted.; 	
      		%goto exit;
		%end;

	%end;
	%else %put %str(N)OTE: Total parameter not provided, thus Total column not displayed.;

	/*******************************************************************************
	 *************processing FreqThreshold cut-off*********************************/

	%let FreqThreshold=&FreqThreshold.;

	%if %length(&FreqThreshold.) %then %do;
		%if %sysevalf(0<&FreqThreshold.) and %sysevalf(&FreqThreshold.<1) %then %do;
			%put %str(N)OTE: With FreqThreshold=&FreqThreshold., the frequency threshold cut-off %sysevalf(&FreqThreshold.*100)%NRBQUOTE(%) within any tretment group is reqested.;
		%end;
		%else %do;
			%put %str(E)RROR: The range for frequency threshold cut-off requires between 0 and 1, e.g., 0.05. The macro call instance halted.; 	
      		%goto exit;
		%end;
	%end;


    /******************************************************************************
    ******************************** END OF BLOCK 1 *******************************
    *******************************************************************************
    ******************************************************************************/


	/******************************************************************************
    *******************************************************************************
    **************************** BLOCK 2: ANALYSIS ********************************
    ******************************************************************************/

	  proc sql;
		create table work.indata as
        select distinct a.* %if %length(&ds.)=2 %then %do; %str(,) b.&TRTxxA., b.&TRTxxAN. %end;  
		from &INDATA. a %if %length(&ds.)=2 %then %do; %str(,) &ADSL. b %end;
		where &WHERE. and &ds_popu. %if %length(&ds.)=2 %then %do; and a.usubjid=b.usubjid %end;;
	  quit;

	  proc sql;
		create table _TRT1(drop=&TRTNVAR.) as
		select distinct "$_TRT" as FMTNAME, &TRTVAR. as START, &TRTVAR. as END, &TRTVAR. as LABEL, "SM" as HLO, &TRTNVAR.
		from work.indata
		order by &TRTNVAR.;
	  quit;

	  %if %sysfunc(indexc(&TOTAL.,%str('#'))) %then %do;     
		%let totaln=%qsysfunc(countc(&total.,%str(#)));
		data _TRT2;
  			set _TRT1;
  			%do i=1 %to %eval(&totaln.+1);   
            	%let total&i.=%scan(&total.,&i.,%str(#));
				if strip(start)="&&total&i." then do;
					label='Total';
					output;
				end;
			%end;
		run;

   	  	%let dsid=%sysfunc(open(work._TRT2)); 
      	%let n=%sysfunc(attrn(&dsid,nobs));     
      	%let rc=%sysfunc(close(&dsid));         

      	%if &n=0 %then %do;
			%put %str(E)RROR: The subtotal not possibly created. The element must be exactly substracted from treatment variable. The macro call halted.;
		 	%goto exit;
	  	%end; 
	  %end;
	  %else %do;                				
	  	data _TRT2;
		  set _TRT1;
		  LABEL='Total';
		run;
	  %end;

	  data _TRT;
		  set _TRT1 _TRT2;
		  FMTNAME="$_TRT";
	  run;

	  proc format lib=work.formats CNTLIN=_TRT;
	  run;

	  data _TRTN;
		set _TRT1(keep=FMTNAME START END LABEL);

		FMTNAME="_TRTN";
		LABEL=_n_;
		TYPE='I';
		output;

		FMTNAME="_TRTN";
		START='Total';
		END='Total';
		LABEL=999;
		TYPE='I';
		output;
	  run;

	proc sort data=_TRTN nodup;
	    by LABEL;
	run;
	  
	proc format lib=work.formats CNTLIN=_TRTN;
	run;
	quit;


	proc format;
      	picture _count
      	low-high="0009";

      	picture _pct(round)
      	low-high="009.9";
	run;

	proc sql;
  		create table _denom as
  		select distinct USUBJID, &TRTxxA., &TRTxxAN. %if %length(&subGroup.) %then %do; %str(,) &subGroup. %end; 
  		from &ADSL.
	  	where &ADSL_popu.='Y'
		%if %length(&subGroup.) %then %do; order by &subGroup. %end;;
	quit;

	proc means data=_denom completetypes n noprint; 
		format &TRTxxA. $_trt.;
		%if %length(&subGroup.) %then %do; BY &subGroup. %end;; 
		class &TRTxxA./mlf preloadfmt order=data;

  		types &TRTxxA.;
  		output out=denom(drop=_:) n=count;
	run; 

	proc sql;
  		create table _TRTA as
  		select distinct USUBJID, &TRTVAR., &TRTNVAR. %if %length(&subGroup.) %then %do; %str(,) &subGroup. %end;  
  		from work.indata 
  		order by %if %length(&subGroup.) %then %do; &subGroup. %str(,) %end; &TRTNVAR.;
	quit;

	proc means data=_TRTA completetypes n noprint; 
		format &TRTVAR. $_trt.;
		%if %length(&subGroup.) %then %do; BY &subGroup. %end;; 
  		class &TRTVAR./mlf preloadfmt order=data;

  		types &TRTVAR.;
  		output out=TRTA_(drop=_:) n=count;
	run; 

    data TRTA;
		length &AVAL1. $120.;
		set TRTA_;
        &AVAL1.="No. of Subjects with Events"; 
	run;


	%do j=1 %to %eval(&an. + 1);

		proc sql;
  			create table _&&AVAL&j. as
 			select distinct usubjid, &TRTVAR., &TRTNVAR., &&sqls_str&j. %if %length(&subGroup.) %then %do; %str(,) &subGroup. %end;
  			from work.indata
			%if %length(&subGroup.) %then %do; 
				order by &subGroup. %if %length(&&sqlby_str&j.) %then %do; %str(,) &&sqlby_str&j. %end;;
			%end;
  			%else %do; 
				%if %length(&&sqlby_str&j.) %then %do; order by &&sqlby_str&j.; %end;
			%end;
		quit;

		proc means data=_&&AVAL&j. completetypes n noprint;   		
			format &TRTVAR. $_trt.;
  			class &TRTVAR./mlf preloadfmt order=data;

			%if %length(&&fmt_str&j.) %then %do;
				format &&fmt_str&j.;
  				class &&class_str&j./mlf preloadfmt order=data;
			%end;

			%if %length(&subGroup.) %then %do; 
				by &subGroup. %if %length(&&by_str&j.) %then &&by_str&j.;; 
			%end;
  			%else %do; 
				%if %length(&&by_str&j.) %then %do; by &&by_str&j.; %end; 
			%end;

  			%if %length(&&typ_str&j.) %then %do; types &&typ_str&j. %str(*) &TRTVAR.; %end; %else types &TRTVAR.;;

  			output out=&&AVAL&j.._(drop=_:) n=count;
		run;

		proc sql;
			create table _&&AVAL&j.._ as
			select distinct a.*, case when a.count>0 then a.count/b.count*100 end as pct, strip(a.&TRTVAR.)||'~(N='||strip(put(b.count, 8.))||')' as TRTDSPL, 
							input(a.&TRTVAR, _TRTN.) as TRTORD							 
			from &&AVAL&j.._ a, denom b
			where %if %length(&subGroup.) %then %do; a.&subGroup.=b.&subGroup. and %end; a.&TRTVAR.=b.&TRTxxA.;
		quit; 

	%end;

	proc sql;
		create table TRTA2 as
		select distinct a.*, input(a.&TRTVAR, _TRTN.) as TRTORD, a.count/b.count*100 as pct, strip(a.&TRTVAR.)||'~(N='||strip(put(b.count, 8.))||')' as TRTDSPL
		from TRTA a, denom b
		where %if %length(&subGroup.) %then %do; a.&subGroup.=b.&subGroup. and %end; a.&TRTVAR.=b.&TRTxxA.;
	quit; 

    /******************************************************************************
    ******************************** END OF BLOCK 2 *******************************
    *******************************************************************************
    ******************************************************************************/

	
    /******************************************************************************
    *******************************************************************************
    **************************** BLOCK 3: FORMATING *******************************
    ******************************************************************************/

    %do j=1 %to %eval(&an. + 1);

	  %let pby&j.=;
	  %let tby&j.=;
	  %let on&j.=;

	  %if &j.>1 %then %do;
		%do i=1 %to %eval(&j.-1); 
			%let pby&j.=&&pby&j. &&AVAL&i. %str(,); 
			%let tby&j.=&&tby&j. &&AVAL&i.;
			%let on&j.=&&on&j.  a.&&AVAL&i.=b.&&AVAL&i. %str(and); 
		%end;
	  %end;

	  %if %length(&&AVALP&j.) %then %do;          
		proc sql;
			create table &&AVAL&j. as
 			select distinct *, input(&&AVAL&j., &&AVALP&j...) as &&AVAL&j..ORD
			from _&&AVAL&j.._
			order by %if %length(&subGroup.) %then %do; &subGroup. %str(,) %end; &&pby&j. &&AVAL&j..ORD, &&AVAL&j.;
		quit;
	  %end; 

	  %else %if %length(&&AVALN&j.) %then %do;  

		proc sql;
		  create table &&AVAL&j. as 
		  select distinct a.*, b.&&AVALN&j.,  b.&&AVALN&j. as &&AVAL&j..ORD
		  from _&&AVAL&j.._ a left join work.indata b
		  on %if %length(&subGroup.) %then %do; a.&subGroup.=b.&subGroup. and %end; a.&&AVAL&j.=b.&&AVAL&j.;
		quit;

	  %end; 

	  %else %if %length(&&AVALI&j.) %then %do;  
		proc sql;
			create table &&AVAL&j. as
 			select distinct *, input(&&AVAL&j., &&AVALI&j.) as &&AVAL&j..ORD   
			from _&&AVAL&j.._
			order by %if %length(&subGroup.) %then %do; &subGroup. %str(,) %end; &&pby&j. &&AVAL&j..ORD, &&AVAL&j.;
		quit;
	  %end; 

	  %else %do;								
		proc sql;
			create table _&&AVAL&j.._T as
 			select distinct *
			from _&&AVAL&j.._
			where strip(&TRTVAR.)="&&SORTRT&j."
			order by %if %length(&subGroup.) %then %do; &subGroup. %str(,) %end; %if %length(&&pby&j.) %then %do; &&pby&j. %end;  pct desc, &&AVAL&j.;
		quit;

		%if %eval(&j.-1)=0 %then %do;      
			data _&&AVAL&j.._T2;
  				set _&&AVAL&j.._T;
  				by %if %length(&subGroup.) %then &subGroup.; &&tby&j. descending PCT &&AVAL&j.;
  				&&AVAL&j..ORD+1;

        		%if %length(&subGroup.) %then %do;
  					if first.&subGroup. then &&AVAL&j..ORD=1;
				%end;

			run;
		%end;
		%else %do;                         
			%let k=%eval(&j.-1);
		 	data _&&AVAL&j.._T2;
  				set _&&AVAL&j.._T;
  				by %if %length(&subGroup.) %then &subGroup.; &&tby&j. descending PCT &&AVAL&j.;
  				&&AVAL&j..ORD+1;
				
  				if first.&&AVAL&k. then &&AVAL&j..ORD=1;
				
			run;
		%end;

		proc sql;
  			create table &&AVAL&j. as
  			select distinct a.*, b.&&AVAL&j..ORD
  			from _&&AVAL&j.._ a left join _&&AVAL&j.._T2 b
  			on %if %length(&subGroup.) %then %do; a.&subGroup.=b.&subGroup. and %end; %if %length(&&on&j.) %then %do; &&on&j. %end; a.&&AVAL&j.=b.&&AVAL&j.;
		quit;

	  %end;	

	%end; 	


    data allnpct;
		set &ANALVAR_str.;
	run;
	proc sort data=allnpct;
		by %if %length(&subGroup.) %then &subGroup.; &ANALVAR_str. TRTORD;
	run;

	data allnpct2;
  		set allnpct;
  		by %if %length(&subGroup.) %then &subGroup.; &ANALVAR_str.;
  		retain %do j=1 %to %eval(&an. + 1); &&AVAL&j..ORD_ %end;; 

		%do j=1 %to %eval(&an. + 1);
  			if &&AVAL&j.^='' then do;
				if &&AVAL&j..ORD>0 then &&AVAL&j..ORD_=&&AVAL&j..ORD;
				else &&AVAL&j..ORD=&&AVAL&j..ORD_;
  			end;
		%end;

  		drop %do j=1 %to %eval(&an. + 1); &&AVAL&j..ORD_ %end;; 
	run;

	%let k=%eval(&an. + 1);
	data all;
		set TRTA2 allnpct2;
	run;
	proc sort data=all;
		by %if %length(&subGroup.) %then &subGroup.; %do j=1 %to %eval(&an. + 1); &&AVAL&j..ORD %end; TRTORD;
	run;

	%if %length(&TOTAL.)=0 %then %do;     
		data total;
			set all;
			if &TRTVAR.='Total' then delete; 
		run;
	%end;
	%else %do;           
		%if %upcase(%sysfunc(compress(&TOTAL.)))=Y or %upcase(%sysfunc(compress(&TOTAL.)))=YES or %sysfunc(indexc(&TOTAL.,%str('#'))) %then %do;  
			data total;
				set all;
			run;
		%end;
		%else %do;            
			%if %upcase(%sysfunc(compress(&TOTAL.)))=N or %upcase(%sysfunc(compress(&TOTAL.)))=NO or %upcase(%sysfunc(compress(&TOTAL.)))=NONE or %upcase(%sysfunc(compress(&TOTAL.)))=NULL %then %do;
				data total;
					set all;
					if &TRTVAR.='Total' then delete; 
				run;
			%end;

		%end;

	%end;

	%if %length(&FreqThreshold.) %then %do;
		proc sql;
			create table trtmax as 
			select distinct *, max(pct) as maxpct
			from total
			group by %if %length(&subGroup.) %then %do; &subGroup. %str(,) %end; &sqls_str.
  			order by %if %length(&subGroup.) %then %do; &subGroup. %str(,) %end; &sqls_str. %str(,) TRTORD;
		quit;

		%let cutoff=%sysevalf(&FreqThreshold.*100);

		data threshold;
			set trtmax;
			where maxpct>&cutoff.;
		run;
	%end;
	%else %do;
		data threshold;
			set total;
		run;
	%end;


	%let lblh=;
	%do j=1 %to %eval(&an. + 1);
 		data _null_;          
	  		set DS_vars;
	  		where NAME="&&AVAL&j.";
	  		call symputx("ANALVARLBL&j.", LABEL);
		run;
		%if &j.=1 %then %let lblh=&&ANALVARLBL&j.; 
		%else %let lblh=&lblh. %str(^n) %sysfunc(repeat(%str(^w),%eval(&j.*2-3))) &&ANALVARLBL&j.; 
	%end;

    %let k=%eval(&an.+1);
    %let COMPUTE=;

    %if &an.=0 %then %let COMPUTE=%str(disp=strip(&&aval&k.)); 
    %else %do; 

		%do i=1 %to %eval(&an.+1);   

        	%if &i.=1 %then %do;    

            	%let stat1=%str(if);
                  
                %do j=1 %TO &k.; 
                	%if &j.=1 %then %let stat1=&stat1. &&aval&j.^=' '; 
                    %else %let stat1=&stat1. %str(and) &&aval&j.=' ' ; 
                %end;
            
                %let stat1=&stat1. %str(then) disp=strip(&&aval&i.); 
                %let COMPUTE=&COMPUTE. &stat1;
            %end;

            %else %if &i.=&k. %then %do; 
	            %let li=%sysfunc(putn(%sysevalf((&i.-1)*220/1440), best4.2))in;
    	        %let stat&k.=%str(else) disp="^S={indent=&li.}"||strip(&&aval&i.); 
                %let COMPUTE=&COMPUTE. %str(;) &&stat&k. %str(;) ;
            %end;

            %else %do; 
            	%let stat&i.=%str(else if);
                %let part1_&i.=;
                %let part2_&i.=;
                %do j=1 %to &k.; 
                	%if &j<=&i. %then %do;
                    	%if &j.=1 %then %let part1_&i.=&&part1_&i. &&aval&j.^=' ';
                        %else %let part1_&i.=&&part1_&i. %str(and) &&aval&j.^=' ';
                    %end;
                    %else %do;
                    	%let part2_&i.=&&part2_&i. %str(and) &&aval&j.=' ';
                    %end;
            	%end;

            	%let stat&i.=&&stat&i. &&part1_&i. &&part2_&i.; 
            	%let li=%sysfunc(putn(%sysevalf((&i.-1)*220/1440), best4.2))in;
            	%let stat&i.=&&stat&i. %str(then) disp="^S={indent=&li.}"||strip(&&aval&i.); 
            	%let COMPUTE=&COMPUTE. %str(;) &&stat&i.;

    		%end;

		%end; 

	%end;;

	data indent;
		length disp $200;
  		set threshold;
 		&compute.;
	run;
	proc sort data=indent;
		by %if %length(&subGroup.) %then &subGroup.; %do j=1 %to %eval(&an. + 1); &&AVAL&j..ORD %end; TRTORD;
	run;

	data reportdata;
		set indent;
		by disp notsorted; 
		retain dispord 0;
		if first.disp then dispord+1; 
	run;


	%if %length(&SubGroup.) %then %do;  
		%let sysfoot1=%str(N: denominator, summarizing the total number of subjects with &popuvar.=Y from ADSL by sub-group &SubGroup..);
	%end;
	%else %let sysfoot1=%str(N: denominator, summarizing the total number of subjects with &popuvar.=Y from ADSL.);

	%let sysfoot2=%str(Sort order: );
	%do j=1 %to %eval(&an. + 1);   

		%if %length(&&class_str&j.) %then %do; 
			%if &j.=1 %then %let sysfoot2=&sysfoot2. &&aval&j. by user-provided format;
			%else %let sysfoot2=&sysfoot2. %str(,) &&aval&j. by user-provided format;
		%end;

		%else %if %length(&&avalN&j.) %then %do; 
			%if &j.=1 %then %let sysfoot2=&sysfoot2. &&aval&j. by existed sort order variable %upcase(&&avalN&j.);
			%else %let sysfoot2=&sysfoot2. %str(,) &&aval&j. by existed sort order variable %upcase(&&avalN&j.);
		%end;

		%else %if %length(&&avalI&j.) %then %do; 
			%if &j.=1 %then %let sysfoot2=&sysfoot2. &&aval&j. by user-provided informat;
			%else %let sysfoot2=&sysfoot2. %str(,) &&aval&j. by user-provided informat;
		%end;

		%else %do; 
			%if &j.=1 %then %let sysfoot2=&sysfoot2. &&aval&j. by descending frequency &&freqtotal&j.;
			%else %let sysfoot2=&sysfoot2. %str(,) &&aval&j. by descending frequency &&freqtotal&j.;
		%end;

	%end;
	%let sysfoot2=&sysfoot2..;

	%let sysfoot3=;
	%if %upcase(&TOTAL.) =Y or %upcase(&TOTAL.) =YES %then %do;                    
		proc sql noprint;
			select distinct START into: sysfoot3 separated by ' + '
			from work._TRT1;
		quit;
	%end;
	%else %if %sysfunc(indexc(&TOTAL.,%str('#'))) %then                            
		%let sysfoot3=%sysfunc(translate(&TOTAL.,  %str( + ), %str(#)));

	%if %length(&sysfoot3.) %then %let sysfoot3=Total: &sysfoot3..;

	%let sysfoot4=;
	%if %length(&FreqThreshold.) %then %do;
	    %if %length(&sysfoot3.) %then %let Totaltext=%str( )including total.; %else %let Totaltext=.;
	    
		%let sysfoot4=The frequency threshold cut-off by %sysevalf(&FreqThreshold.*100)%NRBQUOTE(%) within any tretment group&Totaltext.;;
	%end;


	/******************************************************************************
    ******************************** END OF BLOCK 3 *******************************
    *******************************************************************************
    ******************************************************************************/


    /******************************************************************************
    *******************************************************************************
    ****************************** BLOCK 4: REPORTING *****************************
    ******************************************************************************/

    /*****************************************************************************
    ******************* Store results for RDS ************************************
    *****************************************************************************/
	data rds.&tableNM(label="RDS of &tableNM.");
		set reportdata;
	run;

    /******************************************************************************
    *** Output the final table in RTF format                                    ***
    ******************************************************************************/
    ods results=off;
	ods rtf file="&rtfpath.&tableNM..rtf" style=_MARTF; 
	title1 j=l "Regeneron Pharmaceuticals, Inc."
    	   j=r "Page ^{thispage} of ^{lastpage}";
	%if %symexist(protocol) %then %do; title2 j=l "Product/Protocol: &protocol."; %end;
	title3;

	%do i=1 %to %eval(&tn.+1);
	    %let k=%eval(&i.+3);
		%if &i.=1 %then %do; title4 j=c "%sysfunc(propcase(&tableid.)) &&titl&i."; %end;
		%else %do; title&k. j=c "&&titl&i."; %end;
	%end;

	%if %length(&subGroup.) %then %do;
		%let k=%eval(&tn.+5);
		title&k. j=l "Subgroup: #byvar1=#byval1"; 
	%end;

	proc report data=work.reportdata out=final nowd headline headskip missing split='~' style(lines)=[borderbottomcolor=white] 
				%if %symexist(outputwidth) %then %do; %if %length(&outputwidth.) %then style(report)={outputwidth=&outputwidth.}; %end;;   
		%if %length(&subGroup.) %then %do; by  &subGroup.; %end;
  		column dispord disp (TRTDSPL, ("^R/RTF'\brdrb\brdrs\brdrw15'" (count pct))) dummy; 

		define dispord/group order=internal noprint;
		define disp/group order=data ID FLOW left "&lblh." style(header)=[just=l]; 

		define TRTDSPL/across order=data ''; 
		define count/display "n" format=_count. center FLOW;    
		define pct/display "%"  format=_pct. center FLOW; 
		define dummy /computed noprint;  

  		compute after _page_ /style={just=l};  
			line "&sysFoot1.";
			line "&sysFoot2.";
			%if %length(&sysFoot3.) %then line "&sysFoot3.";;
			%if %length(&sysFoot4.) %then line "&sysFoot4.";;

			%if %length(&footnote.) %then %do;
				%do i=1 %to %eval(&fn.+1);
  					line "&&foot&i.";
 				%end; 
			%end;
  		endcomp;	
		
		%if &sysscp=WIN %then %do;
  			footnote10 j=l color=blue "Creation Program: &progpathnm."  j=r  "%sysfunc(date(),date9.) %sysfunc(time(),tod8.) [user:&sysuserid.]";
		%end;
		%else %do;
			footnote10 j=l color=blue "Creation Program: %sysfunc(dequote(&_clienttasklabel.)).sas" j=r "%sysfunc(date(),date9.) %sysfunc(time(),tod8.) [user:&sysuserid.]";
		%end;
	run;

	ods rtf close;


    /******************************************************************************
    ******************************** END OF BLOCK 4 *******************************
    *******************************************************************************
    ******************************************************************************/


    /******************************************************************************
    *******************************************************************************
    ****************************** BLOCK 5: CLEAN UP ******************************
    ******************************************************************************/

	title;
	footnote;

	proc template;
		delete Styles._MARTF;
	run;

	%if %sysfunc(libname(rds))=0 %then %put NOTE: libname RDS cleared;

	%if %symexist(debug)=0 %then %do;  /*macro variable DEBUG not found*/ 
		%if &dsn. %then %do; 
			proc datasets library=work nolist memtype=data;
   				save &dslist. ;
			run;
			quit;
		%end;
		%else %do;     
			proc datasets lib=work nolist kill force memtype=data;
				run;
			quit;
		%end;
	%end;
	%else %do;      
		%if %upcase(&debug.)^=Y and %upcase(&debug.)^=YES %then %do;
		  %if &dsn. %then %do; 
			proc datasets library=work nolist memtype=data;
   				save &dslist. ;
			run;
			quit;
		  %end;
		  %else %do;     
			proc datasets lib=work nolist kill force memtype=data;
				run;
			quit;
		  %end;		
	    %end;
		%else %do;
			%put;
			%put %str(N)OTE: The macro in DEBUGING mode, please check the intermediate datasets in SAS WORK libref and LOGs.; 
			%put;
		%end;
	%end;

	
	%let pfmt_np=; 
	%do j=1 %to %eval(&an.+1); 
	    %if %length(&&AVALP&j.) %then %let pfmt_np=&pfmt_np. &&AVALP&j.;
	%end;	

    proc catalog catalog=work.formats;
		delete _trtn &pfmt_np./et=infmt;
        delete _count _pct /entrytype=format;
        delete  _TRT/entrytype=formatc;;
    run;
    quit;

	ods results=on;
	ods listing;
	ods html; 

    /******************************************************************************
    ******************************** END OF BLOCK 5 *******************************
    *******************************************************************************
    ******************************************************************************/

	%exit:

	proc printto;
	run;

	data _null_; 
      infile logfile truncover expandtabs; 
      input @1 newstring $char256.;
      newstring = tranwrd(newstring,'ERROR ','ERROR: '); /*keeping ERROR still replayed in Red with colon*/
      put @1 newstring $char256.; 
	run; 
	options source;

    /******************************************************************************
    ******************************** THE END **************************************
    ******************************************************************************/

%mend PSBJNPCT1;

